/************************************************************************
	main.c

    USB RGB Bargraph
    Copyright (C) 2010 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

#ifndef MAIN_C
#define MAIN_C

// Global includes
#include <htc.h>

// Local includes
#include "hardware.h"
#include "tlc5940.h"
#include "rgbled.h"

// USB Stack includes
#include "usb.h"
#include "HardwareProfile.h"
#include "usb_function_hid.h"
#include "genericHID.h"

// PIC 18F2550 fuse configuration:
// Config word 1 (Oscillator configuration)
// 20Mhz crystal input scaled to 48Mhz and configured for USB operation
__CONFIG(1, USBPLL & IESODIS & FCMDIS & HSPLL & CPUDIV1 & PLLDIV5);
// Config word 2
__CONFIG(2, VREGEN & PWRTDIS & BOREN & BORV20 & WDTDIS & WDTPS32K);
// Config word 3
__CONFIG(3, PBDIGITAL & LPT1DIS & MCLREN);
// Config word 4
__CONFIG(4, XINSTDIS & STVREN & LVPDIS & ICPORTDIS & DEBUGDIS);
// Config word 5, 6 and 7 (protection configuration)
__CONFIG(5, UNPROTECT);
__CONFIG(6, UNPROTECT);
__CONFIG(7, UNPROTECT);

// Process USB input and output
void processUsbIO(void)
{   
    // If we are not in the configured state just return
    if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1))
	    return;

	// Check if data was received from the host.
    if(!HIDRxHandleBusy(USBOutHandle))
    {
	    // Process received USB commands
	    
	    // We support two commands:
	    // 0x80 - Set the brightness levels of all 16 RGB LEDs.  Expects 0-255 red, green and blue
	    // bytes.
	    //
	    // 0x81 - Set the fade on and fade off levels of all 16 RGB LEDs. Expects 2 bytes per LED
	    // fade-on speed and fade-off speed 0-255 (time is measured in milliseconds).
	          
        switch(ReceivedDataBuffer[0])
		{
            case 0x80:  // Command 0x80 - Set LED brightness levels
            	// void setRgbLedColour(unsigned char rgbLedNumber, int red, int green, int blue);
            	// Write LED brightness information to the bargraph
            	for (unsigned char ledNumber = 0; ledNumber < 16; ledNumber++)
            	{
	            	unsigned char pointer = (3 * ledNumber) + 1;
	            	
	            	// Set the colours and convert the values from 0-255 to 0-4095
	            	setRgbLedColour(ledNumber,
	            		(int)ReceivedDataBuffer[pointer]*16,
	            		(int)ReceivedDataBuffer[pointer+1]*16,
	            		(int)ReceivedDataBuffer[pointer+2]*16);
	            }	
            	break;
            
            case 0x81:	// Command 0x81 - Set LED fade off speeds
				// void setRgbLedFadeSpeed(unsigned char rgbLedNumber, int fadeOn, int fadeOff);
				// Write LED fade speed information to the bargraph
				for (unsigned char ledNumber = 0; ledNumber < 16; ledNumber++)
            	{
	            	setRgbLedFadeSpeed(ledNumber, 0, (int)ReceivedDataBuffer[ledNumber + 1]);	
	            }
            	break;
            	
            default:	// Unknown command received
           		break;
		}
		
        // Re-arm the OUT endpoint for the next packet
        USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&ReceivedDataBuffer,64);
    }  
} 

// Main function
void main(void)
{
	// General PIC initialisation ---------------------------------------------
	
	// Configure ports as inputs (1) or outputs(0)
	TRISA = 0b00000000;
	TRISB = 0b00000000;
	TRISC = 0b00000000;

	// Clear/Set ports
	PORTA = 0b00000000;
	PORTB = 0b00000000;
	PORTC = 0b00000000;

	// Turn off analogue ports
	ADCON1 = 0x0F;	

	// USB set up -------------------------------------------------------------

	// USB stack and parameter initialisation
    #if defined(USE_SELF_POWER_SENSE_IO)
    tris_self_power = INPUT_PIN;
    #endif
    
    // Initialize the variable holding the handle for the last
    // transmission
    USBOutHandle = 0;
    USBInHandle = 0;
    
	// Call the stack initialisation functions
    USBDeviceInit();
    
    // ------------------------------------------------------------------------

	// Initialise the RGB Display
	initialiseRgbDisplay();
	
	// Set the initial LED fade speeds
	for (int rgbLedNumber = 0; rgbLedNumber < 16; rgbLedNumber++)
		setRgbLedFadeSpeed(rgbLedNumber, 0, 0);
		
	// Main loop
	while(1)
	{
		// Check bus status and service USB interrupts.
        USBDeviceTasks();

		// Application-specific USB tasks.
        processUsbIO();    
	}
}

#endif